#include <stdio.h>
#include <assert.h>
#include "csv.h"
#include "api.h"
#include <string.h>


#define FILE_READ_BUFFER_SIZE 2048

// Get the API version information
const char* api_version() {
    return "UOC PP 20211";
}

// Load data from a CSV file. If reset is true, remove previous data
tApiError api_loadData(tApiData* data, const char* filename, bool reset) {
    tApiError error;
    FILE *fin;    
    char buffer[FILE_READ_BUFFER_SIZE];
    tCSVEntry entry;
    
    // Check input data
    assert( data != NULL );
    assert(filename != NULL);
    
    // Reset current data    
    if (reset) {
        // Remove previous information
        error = api_freeData(data);
        if (error != E_SUCCESS) {
            return error;
        }
        
        // Initialize the data
        error = api_initData(data);
        if (error != E_SUCCESS) {
            return error;
        }
    }

    // Open the input file
    fin = fopen(filename, "r");
    if (fin == NULL) {
        return E_FILE_NOT_FOUND;
    }
    
    // Read file line by line
    while (fgets(buffer, FILE_READ_BUFFER_SIZE, fin)) {
        // Remove new line character     
        buffer[strcspn(buffer, "\n\r")] = '\0';
        
        csv_initEntry(&entry);
        csv_parseEntry(&entry, buffer, NULL);
        // Add this new entry to the api Data
        error = api_addDataEntry(data, entry);
        if (error != E_SUCCESS) {
            return error;
        }
    }
    
    fclose(fin);
    
    return E_SUCCESS;
}

// Add a new entry
tApiError api_addDataEntry(tApiData* data, tCSVEntry entry) { 
    
    //////////////////////////////////
    // Ex PR1 2
    /////////////////////////////////
    
    if (data == NULL) {
        api_initData(data);
    }
    
    // Managing PERSON type
    if (strcmp(csv_getType(&entry), "PERSON") == 0) {
        
        // Check if numFields is the expected; return INVALID_ENTRY_FORMAT otherwise
        if (csv_numFields(entry) == 7) {
        
            tPerson newPerson;
            person_parse(&newPerson,entry);
            
            // Check if person already exists; return DUPLICATED_ENTRY otherwise
            if (population_find(data->population, newPerson.document) == -1) {
                
                // Add person to population variable inside ApiData
                population_add(&data->population, newPerson);
                return E_SUCCESS;
                
            } else {
                return E_DUPLICATED_ENTRY;
            }
            
        } else {
            return E_INVALID_ENTRY_FORMAT;
        }
        
    } else if (strcmp(csv_getType(&entry), "GEOLOCATION") == 0) {
        
        // Check if numFields is the expected; return INVALID_ENTRY_FORMAT otherwise
        if (csv_numFields(entry) == 5) {
            
            //tGeolocationData newGeolocationData;
            tTimePosition newTimePosition;
            contact_parse(&newTimePosition, entry);
            
            // Check if record does not exist yet; return DUPLICATED_ENTRY otherwise
            if(geoData_find(data->geolocationData, newTimePosition.person, newTimePosition.timestamp) == -1) {
                
                // Add geolocation info for variable inside ApiData
                geoData_add(&data->geolocationData, newTimePosition);
                
            } else {
                return E_DUPLICATED_ENTRY;
            }
            
        } else {
            return E_INVALID_ENTRY_FORMAT;
        }

    } else {
        return E_INVALID_ENTRY_TYPE;
    }
    
    //return E_NOT_IMPLEMENTED;
    return E_SUCCESS;
}

// Free all used memory
tApiError api_freeData(tApiData* data) {
    //////////////////////////////////
    // Ex PR1 2: 
    /////////////////////////////////
    
    assert(data != NULL);

    population_free(&data->population);
    geoData_free(&data->geolocationData);
    
    //return E_NOT_IMPLEMENTED;
    return E_SUCCESS;
}

// Initialize the data structure
tApiError api_initData(tApiData* data) {
        
    //////////////////////////////////
    // Ex PR1 2: 
    /////////////////////////////////
    
    assert(data != NULL);
    
    geoData_init(&data->geolocationData);
    population_init(&data->population);
    
    //return E_NOT_IMPLEMENTED;
    return E_SUCCESS;
}

// Get person data
tApiError api_findPerson(tApiData* data, const char* document, tCSVEntry* person) {
    //////////////////////////////////
    // Ex PR1 3
    /////////////////////////////////

    // Init new person entry for CSV data storage
    csv_initEntry(person);
    
    int index = population_find(data->population, document);    
    
    if (index != -1) {
        
        char strCSV[FILE_READ_BUFFER_SIZE];
        
        tPerson found = data->population.elems[index];
        
        // Fill strCSV variable with data from the person found
        sprintf(strCSV, "%s;%s;%s;%s;%s;%s;%s;%d/%d/%d",
            "PERSON", found.document, found.name, found.surname, 
            found.email, found.address, found.cp,
            found.birthday.day, found.birthday.month, found.birthday.year);

        csv_parseEntry(person, strCSV, NULL);
            
        return E_SUCCESS;
        
    } else {
        return E_PERSON_NOT_FOUND;
    }
    
}

// Get all geolocation data for a given person
tApiError api_getPersonGeolocation(tApiData* data, const char* document, tCSVData* geodata) {
    //////////////////////////////////
    // Ex PR1 3
    /////////////////////////////////    
    
    assert(geodata != NULL);
    tApiError ret;
    
    csv_init(geodata);
    
    int i;
    for(i = 0; i < data->geolocationData.count; i++) {
        if(strcmp(data->geolocationData.elems[i].person, document) == 0) {
            
            tTimePosition found = data->geolocationData.elems[i];
            tCSVEntry csvEntry;
            char strCSV[FILE_READ_BUFFER_SIZE];
            
            // Fill strCSV variable with data from the person found
            sprintf(strCSV, "%s;%s;%d/%d/%d;%d:%d;%f;%f",
                "GEOLOCATION", found.person,
                found.timestamp.date.day, found.timestamp.date.month, found.timestamp.date.year,
                found.timestamp.time.hour, found.timestamp.time.minutes,
                found.coordinate.latitude, found.coordinate.longitude);

            csv_addStrEntry(geodata, strCSV, NULL);
        }
    }
    
    if (geodata->count != 0) {
        return E_SUCCESS;
    } else {
        return E_PERSON_NOT_FOUND;
    }

}

// Remove a person
tApiError api_removePerson(tApiData* data, const char* document) {
    //////////////////////////////////
    // Ex PR1 3
    ///////////////////////////////// 

    population_del(&data->population, document);
    
    tCSVData toDelete;
    tDateTime dateTime;
    
    char date[11];
    char time[6];
    
    api_getPersonGeolocation(data, document, &toDelete);
    
    for(int i = 0; i < toDelete.count; i++) {
        
        csv_getAsString(toDelete.entries[i], 1, date, 11);
        csv_getAsString(toDelete.entries[i], 2, time, 6);
        
        dateTime_parse(&dateTime, date, time);
        
        geoData_del(&data->geolocationData, document, dateTime);
    }
    
    if (toDelete.count != 0) {
        return E_SUCCESS;
    } else {
        return E_PERSON_NOT_FOUND;
    }
    
}
